# coding=utf-8
from django.conf import settings
from django.http.response import Http404


class Middleware(object):

    def process_response(self, request, response):
        # ignore all ajax, static and media requests
        path = request.path_info
        if request.is_ajax() or \
                path.startswith(settings.STATIC_URL or '///') or \
                path.startswith(settings.MEDIA_URL or '///'):
            return response

        if response.status_code != 404:
            return response

        from .views import cms_page_index
        # we return the original response
        # if the 404 response is given by cms_page_index
        if getattr(request.resolver_match, 'func', None) is cms_page_index:
            return response

        try:
            return cms_page_index(request)
        except Http404:
            return response
        except Exception:
            if settings.DEBUG:
                raise
            return response



# coding=utf-8
from django.conf import settings
from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^cms/ajax/save/draft/$', views.save,
        name='save_cms_content', kwargs={'save_type': 'draft'}),
    url(r'^cms/ajax/save/publish/$', views.save,
        name='publish_cms_content', kwargs={'save_type': 'publish'}),
    url(r'^.+{}|^$'.format(r'/$' if settings.APPEND_SLASH else r''),
        views.cms_page_index, name='cms_index'),
]



# coding=utf-8
import json

from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.core.serializers.json import DjangoJSONEncoder
from django.http.response import Http404, HttpResponse
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.views.decorators.csrf import csrf_protect

from .conf import settings
from .models import Content
from .utils import NAMESPACE_DELIMITER, dum_request, parse_cms_template
from .validators import validate_and_get_template

_template_not_defined = object()


def render(request, template_name=_template_not_defined,
           template_context=None, content_type=None, status=None,
           using=None, cms_context=None):
    """
    If request is given:
    Checks if an entry exists in the Content model
    for the request path and renders it.
    If it fails with 404 and a template_name is given,
    then it falls back to render_to_response with the given arguments.

    If request is not passed:
    render_to_response is called with the given arguments.

    If render_to_response fails with ValueError or Http404,
     then it will be rendered without cms_context.
    """
    if request is not None:
        try:
            cms_page = Content.objects.get(url=request.path_info)
        except Content.DoesNotExist:
            pass
        else:
            try:
                return render_content(
                    cms_page=cms_page,
                    request=request,
                    template_context=template_context,
                    content_type=content_type,
                    status=status,
                    using=using
                )
            except (TemplateDoesNotExist, Http404):
                if settings.DEBUG:
                    raise

    if template_name is _template_not_defined:
        raise Http404('Backup template name is not defined')

    try:
        return render_to_response(
            template_name=template_name,
            template_context=template_context,
            content_type=content_type,
            status=status,
            using=using,
            request=request,
            cms_context=cms_context
        )
    except (ValueError, Http404):
        if settings.DEBUG:
            raise
        else:
            return render_to_response(
                template_name=template_name,
                template_context=template_context,
                content_type=content_type,
                status=status,
                using=using,
                request=request
            )


def render_to_response(template_name,
                       template_context=None, content_type=None, status=None,
                       using=None, request=None, cms_context=None, public=True):

    """
    This is similar to the django's render_to_response
    and accepts all of it's non deprecated arguments.
    It also takes two additional parameters request and cms_context.

    It will render the given template with given template_context, cms_context
    and returns corresponding response
    or raises Http404 if the parsing fails for the given cms_context.

    :rtype: HttpResponse

    :return: :raise ValueError:
    """
    @csrf_protect
    def _render(_request):
        template = validate_and_get_template(template_name, using=using)
        content = template.render(context=template_context, request=_request)

        if cms_context is not None:
            if isinstance(cms_context, dict):
                try:
                    content = parse_cms_template(
                        html=content, cms_context=cms_context,
                        public=public, request=_request,
                        template_context=template_context
                    )
                except (ValidationError, TemplateSyntaxError) as e:
                    if settings.DEBUG:
                        raise
                    raise Http404(e)
            else:
                raise ValueError('cms_context should be an instance of dict')

        return HttpResponse(content, content_type, status)

    # if you wish to know why the view
    # or other render functions are not decorated with csrf_protect
    # read the comment in django.contrib.flatpages.views
    if request is None:
        # since we are using csrf_token decorator
        # it expects the first argument of the function to be a request object
        # and we use this dum_request to render the template
        # instead of passing None
        # so as to load the csrf_token inside the template
        request = dum_request
    return _render(request)


def render_content(cms_page,
                   request=None, template_context=None,
                   content_type=None, status=None, using=None):
    """
    Renders a given Content object and returns the HttpResponse
    or will raise Http404 if any syntax or validation errors occur.

    The response object is processed with auth_data or public_data according to the given request.

    Since we are passing an instance of Content object, this function might raise TemplateDoesNotExist
    for the template associated with the Content instance.

    :type cms_page: Content

    :type request: HttpRequest | None

    :type template_context: dict | None

    :rtype: HttpResponse

    :return: :raise Http404:
    """
    template_context = template_context or {}
    template_context['cms_content'] = cms_page

    if request is not None and settings.CFBLOG_CAN_EDIT(request.user):
        editor_context = {
            'view': 'author',
            'cms_data': json.dumps(cms_page.auth_data),
            'cms_page_id': cms_page.id,
            'modified_on': json.dumps(
                cms_page.modified_on, cls=DjangoJSONEncoder
            ),
            'namespace_delimiter': json.dumps(NAMESPACE_DELIMITER),
            'can_publish': settings.CFBLOG_CAN_PUBLISH(request.user),
        }

        cache.set(
            key='template_context_{}'.format(cms_page.id),
            value=template_context,
            timeout=24 * 60 * 60
        )
        template_context.update(editor_context)

        return render_to_response(
            template_name=cms_page.template,
            template_context=template_context,
            content_type=content_type,
            status=status,
            using=using,
            request=request,
            cms_context=cms_page.auth_data,
            public=False
        )

    if cms_page.is_public:
        return render_to_response(
            template_name=cms_page.template,
            template_context=template_context,
            content_type=content_type,
            status=status,
            using=using,
            request=request,
            cms_context=cms_page.public_data,
            public=True
        )

    raise Http404('Page not found')



# coding=utf-8
from __future__ import unicode_literals

import re

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.template.loader import (TemplateDoesNotExist, get_template,
                                    select_template)
from django.utils.lru_cache import lru_cache
from django.utils.translation import ugettext_lazy as _


@lru_cache()
def validate_and_get_template(name, using=None):
    if isinstance(name, (list, tuple)):
        return select_template(name, using=using)
    try:
        if not name.endswith('.html'):
            name = '{}.html'.format(name)
        return get_template(name, using=using)
    except TemplateDoesNotExist as e:
        raise ValidationError(_("Unable to find the template '{}'".format(e)))


url_path_re = re.compile(r'^/(?:[-a-zA-Z0-9_]+/{})*$'.format(r'?' if not settings.APPEND_SLASH else r''))
validate_url_path = RegexValidator(
    url_path_re,
    _("Enter a valid 'url path'. Path should start and end with '/'."),
    'invalid'
)



# coding=utf-8
import json
import traceback

from django.core.cache import cache
from django.http.response import HttpResponseForbidden, JsonResponse
from django.shortcuts import get_object_or_404
from django.utils.dateparse import parse_datetime
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.http import require_POST

from .conf import settings
from .models import Content
from .response import render, render_content
from .signals import post_publish_signal, pre_publish_signal
from .utils import user_passes_test


def cms_page_index(request):
    return render(request)


@require_POST
@csrf_protect
def save(request, save_type):
    if not request.is_ajax():
        return HttpResponseForbidden()

    if save_type not in ('draft', 'publish'):
        return JsonResponse({'success': False}, status=400)

    if save_type == 'draft':
        test_func = settings.CFBLOG_CAN_EDIT
    else:
        test_func = settings.CFBLOG_CAN_PUBLISH

    return user_passes_test(test_func=test_func)(_save)(request, save_type)


def _save(request, save_type):
    post_data = request.POST

    if any(_ not in post_data for _ in ('auth_data', 'cms_page_id')):
        return JsonResponse({'success': False}, status=400)

    cms_page = get_object_or_404(Content, id=post_data['cms_page_id'])
    draft_date = parse_datetime(post_data['draft_modified'])

    if draft_date and draft_date < cms_page.modified_on:
        return JsonResponse(
            {
                'success': False,
                'draft_error': True,
                'message': 'Draft data was out of date'
            },
            status=200
        )

    try:
        content = json.loads(post_data['auth_data'])
    except:
        return JsonResponse(
            {
                'success': False,
                'message': 'Invalid JSON object'
            },
            status=400
         )
    else:
        cms_page.auth_data.update(content)
        try:
            template_context = cache.get(
                'template_context_{}'.format(cms_page.id)
            )
            render_content(cms_page, request=request,
                           template_context=template_context)
        except Exception as e:
            return JsonResponse(
                {'success': False,
                 'message': 'Unable to parse the new content.\n'
                 'Please check the console for issues.',
                 'exception': unicode(e),
                 'traceback': traceback.format_exc()}
            )
        else:
            if save_type == 'draft':
                cms_page.save()
            else:
                pre_signal_response = pre_publish_signal.send(
                    sender=cms_page._meta.model,
                    cms_page=cms_page
                )
                errors, warns = [], []
                for _, response in pre_signal_response:
                    should_publish, msg = response
                    if should_publish is False:
                        errors.append(msg)
                    elif should_publish is None:
                        warns.append(msg)

                if errors:
                    return JsonResponse(
                        {
                            'success': False,
                            'message_in_detail': '\n'.join(errors),
                            'message': """
                                Unable to publish.
                                Please check console for details
                            """
                        }
                    )
                else:
                    cms_page.publish_cms_content()
                    post_publish_signal.send(
                        sender=cms_page._meta.model,
                        cms_page=cms_page
                    )

                if warns:
                    return JsonResponse(
                        {
                            'success': None,
                            'message_in_details': '\n'.join(warns),
                            'message': """
                                Please check console for warnings
                            """
                        }
                    )

            return JsonResponse({'success': True})



# coding=utf-8
from django.db import models
from django.db.models.query import QuerySet
from django.utils import timezone


class ContentMixin(object):
    def by_author(self, user):
        return self.filter(author=user)

    def by_category(self, category):
        return self.filter(category=category)

    def published(self):
        return self.filter(
            status__gte=self.model.PUBLIC, publish__lte=timezone.now()
        )

    def static_pages(self):
        return self.filter(category__is_static=True)

    def blog_posts(self):
        return self.filter(category__is_static=False)


class ContentQuerySet(QuerySet, ContentMixin):
    pass


class ContentManager(models.Manager, ContentMixin):
    def get_queryset(self):
        return ContentQuerySet(self.model, using=self._db)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
import re
import cfblog.validators
import tagging.fields
import jsonfield.fields
from django.conf import settings
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(unique=True, max_length=100, verbose_name='title', db_index=True)),
                ('description', models.TextField(verbose_name='description')),
            ],
            options={
                'ordering': ('title',),
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.CharField(db_index=True, unique=True, max_length=255, verbose_name='url path', validators=[django.core.validators.RegexValidator(re.compile('^/(?:[-a-zA-Z0-9_]+/)*$'), "Enter a valid 'url path'. Path should start and end with '/'.", 'invalid')])),
                ('template', models.CharField(max_length=255, verbose_name='template', validators=[cfblog.validators.validate_and_get_template])),
                ('status', models.IntegerField(default=1, verbose_name='status', choices=[(1, 'Draft'), (2, 'Public')])),
                ('publish', models.DateTimeField(default=datetime.datetime.now, verbose_name='publish')),
                ('title', models.CharField(max_length=200, verbose_name='title')),
                ('tags', tagging.fields.TagField(max_length=255, blank=True)),
                ('tease', models.TextField(help_text='Concise text suggested. Does not appear in RSS feed.', verbose_name='tease', blank=True)),
                ('notes_todos', models.TextField(verbose_name='Notes and Todos', blank=True)),
                ('auth_data', jsonfield.fields.JSONField(default=dict, help_text="Don't edit this unless you know what this means", verbose_name='author data')),
                ('public_data', jsonfield.fields.JSONField(default=dict, help_text="Don't edit this unless you know what this means", verbose_name='public data')),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name='created on')),
                ('modified_on', models.DateTimeField(auto_now=True, verbose_name='modified on')),
                ('author', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
                ('category', models.ForeignKey(to='cfblog.Category')),
            ],
            options={
                'ordering': ('-publish',),
                'get_latest_by': 'publish',
            },
        ),
    ]



# coding=utf-8
from django.dispatch import Signal

"""
    pre_publish_signal handlers should return two values.
    Frist value should be boolean or None
    True for Success
    False for Errors
    None for Warnings

    Second value should be the corresponding message.
    e.g. [(True, "Success",), (False, "Invalid html",)...]
"""
pre_publish_signal = Signal(providing_args=["cms_page"])
post_publish_signal = Signal(providing_args=["cms_page"])



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-08-08 10:45
from __future__ import unicode_literals

from django.db import migrations, models


def set_static_category(apps, schema_editor):
    Category = apps.get_model('cfblog', 'Category')
    try:
        cat = Category.objects.get(id=1, title='Static Page')
    except Category.DoesNotExist:
        pass
    else:
        cat.is_static = True
        cat.save()


class Migration(migrations.Migration):

    dependencies = [
        ('cfblog', '0005_auto_20160406_1825'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='is_static',
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.RunPython(
            code=set_static_category,
            reverse_code=migrations.RunPython.noop,
        ),
    ]



# coding=utf-8
import re
from functools import wraps

from bs4 import BeautifulSoup, Tag
from django.conf import settings
from django.core import urlresolvers
from django.core.exceptions import PermissionDenied
from django.core.handlers.wsgi import WSGIRequest
from django.template import TemplateSyntaxError
from django.test.client import Client
from django.utils.decorators import available_attrs
from django.utils.functional import SimpleLazyObject
from mistune import markdown

from .validators import ValidationError, validate_and_get_template

# we are initialising Client as global variable
# so that we need to load the middlewares only once
_dum_client = Client()


@SimpleLazyObject
def dum_request():
    """
    Returns WSGIRequest object that is ready to be passed into a view
    """
    request_handler = _dum_client.handler
    environ = _dum_client._base_environ()

    request = WSGIRequest(environ)
    if request_handler._request_middleware is None:
        request_handler.load_middleware()

    urlconf = settings.ROOT_URLCONF
    urlresolvers.set_urlconf(urlconf)
    resolver = urlresolvers.RegexURLResolver(r'^/', urlconf)
    try:
        response = None
        # Apply request middleware
        for middleware_method in request_handler._request_middleware:
            response = middleware_method(request)
            if response:
                break

        if response is None:
            if hasattr(request, 'urlconf'):
                # Reset url resolver with a custom urlconf.
                urlconf = request.urlconf
                urlresolvers.set_urlconf(urlconf)
                resolver = urlresolvers.RegexURLResolver(r'^/', urlconf)

            resolver_match = resolver.resolve(request.path_info)
            callback, callback_args, callback_kwargs = resolver_match
            request.resolver_match = resolver_match

            # Apply view middleware
            for middleware_method in request_handler._view_middleware:
                response = middleware_method(
                    request, callback, callback_args, callback_kwargs
                )
                if response:
                    break
    except:
        pass
    finally:
        return request

ATTR_TAG = 'data-cms-attr'
INCLUDE_TAG = 'data-cms-include'
CONTENT_TAG = 'data-cms-content'
NAMESPACE_TAG = 'data-cms-namespace'
REPLACE_TAG = 'data-cms-replace'

CMS_ATTRIBUTES = [ATTR_TAG, CONTENT_TAG, INCLUDE_TAG]

attr_re = re.compile(
    r'^(?:(?:[a-z-]+:[a-z_0-9]+)+(?:\|(?:[a-z-]+:[a-z_0-9]+))*)*$',
    re.IGNORECASE
)
content_re = re.compile(
    r'^(?:md:)?([a-z_0-9])+$',
    re.IGNORECASE
)
include_html_re = re.compile(
    r'^(?:(?:[a-z_0-9])+:)?.+$',
    re.IGNORECASE
)
namespace_re = re.compile(
    r'^(?:[a-z_0-9])+$',
    re.IGNORECASE
)

NAMESPACE_DELIMITER = '-'

# only 'html.parser' doesn't bother adding <html> tag for html components
HTML_PARSER = 'html.parser'


def parse_cms_template(html, cms_context, parent_namespace='', public=False,
                       request=dum_request, template_context=None):
    """
    Refer to tests for cms syntax

    :param html: Html to be parsed using cms syntax
    :type html: str
    :param cms_context: Dictionary that is to be used to parse the
    cms attributes in template
    :type cms_context: dict
    :param parent_namespace: Namespace of the html content to be parsed (if any)
    :type parent_namespace: str
    :param public: Renders the page for public usage
    :type public: bool
    :param request: Request object to be used for template context
    :param template_context: Template context to be used for rendering the
    base and included templates
    :type template_context: dict
    :rtype : str
    """
    soup = BeautifulSoup(html, features=HTML_PARSER)

    for tag in soup.find_all(attrs={INCLUDE_TAG: include_html_re}):
        namespace = get_namespace(tag, parent_namespace=parent_namespace)
        include_value = tag.attrs.pop(INCLUDE_TAG)
        if ':' in include_value:
            local_namespace, default_template_name = include_value.split(':', 1)
        else:
            try:
                local_namespace = tag.attrs[NAMESPACE_TAG]
            except KeyError:
                raise TemplateSyntaxError(
                    'value of data-cms-include should be of the form '
                    '{namespace}:{template path}.'
                    'if namespace is not specified then another attribute '
                    'data-cms-namespace should be defined'
                )
            else:
                if not namespace_re.match(local_namespace):
                    raise TemplateSyntaxError(
                        '"{}" is not a valid value for {}'.format(
                            local_namespace, NAMESPACE_TAG
                        )
                    )
                else:
                    default_template_name = include_value

        if namespace:
            namespace += NAMESPACE_DELIMITER + local_namespace
        else:
            namespace = local_namespace

        template_name = cms_context.get(namespace, default_template_name)

        if template_name.endswith('.html'):
            template_name = template_name[:-5]

        try:
            include_template = validate_and_get_template(template_name)
        except ValidationError:
            include_template = validate_and_get_template(default_template_name)

        include_html = include_template.render(template_context, request)

        tag.attrs[NAMESPACE_TAG] = local_namespace
        if not public:
            tag.attrs[INCLUDE_TAG] = template_name

        replace_tag_content(tag=tag, content=include_html)

    # soup does not recognize the changes made in above loop unless I do this
    # Also do not move it inside the loop. It will mess up the variable scoping

    for tag in soup.find_all(attrs={ATTR_TAG: attr_re}):
        _ns = get_namespace(tag, parent_namespace=parent_namespace)
        attrs = tag[ATTR_TAG].split('|')

        for attr in attrs:
            attr_name, key = attr.split(':', 1)
            key = _ns + NAMESPACE_DELIMITER + key if _ns else key

            if key in cms_context:
                tag[attr_name] = cms_context[key]

    for tag in soup.find_all(attrs={CONTENT_TAG: content_re}):
        _ns = get_namespace(tag, parent_namespace=parent_namespace)
        key = tag[CONTENT_TAG]
        md = False

        if key.startswith('md:'):
            key = key[3:]
            md = True

        key = _ns + NAMESPACE_DELIMITER + key if _ns else key

        if key in cms_context or REPLACE_TAG in tag.attrs:
            # REPLACE_TAG will be replaced with it's content.
            # So, it doesn't make much sense to process it in else loop
            content = cms_context.get(key, '')
        else:
            content = tag.encode_contents()
            if not any(_ in content for _ in CMS_ATTRIBUTES):
                continue

        if any(_ in content for _ in CMS_ATTRIBUTES):
            content = parse_cms_template(
                html=content, cms_context=cms_context, parent_namespace=key,
                request=request, template_context=template_context
            )

        if md:
            content = markdown(content, False)

        if public and REPLACE_TAG in tag.attrs:
            new_tag = BeautifulSoup(content, features=HTML_PARSER)
            tag.replace_with(new_tag)
        else:
            # We don't replace the tag in auth render so as to keep it editable
            replace_tag_content(tag=tag, content=content)

    # don't use soup.prettify as it will insert empty spaces inside textarea
    return soup.encode_contents()


def replace_tag_content(tag, content):
    tag.clear()

    if isinstance(content, basestring):
        # Don't even bother
        if not content:
            return tag
        content = BeautifulSoup(content, features=HTML_PARSER)

    tag.append(content)
    if content.contents:
        # Not sure why .append doesn't do this by default
        tag.next_element = content.contents[0]
    return tag


def get_namespace(tag, parent_namespace=''):
    ancestor_namespaces = [
        parent.attrs.get(NAMESPACE_TAG) or parent.attrs.get(CONTENT_TAG)
        for parent in tag.find_parents(is_namespace_parent)
    ]

    if parent_namespace:
        ancestor_namespaces.append(parent_namespace)

    return NAMESPACE_DELIMITER.join(reversed(ancestor_namespaces))


def is_namespace_parent(tag):
    attrs = tag.attrs
    if namespace_re.match(attrs.get(NAMESPACE_TAG, '')):
        return True

    if content_re.match(attrs.get(CONTENT_TAG, '')):
        return True

    return False


def can_edit_content(user):
    return any(user.has_perm(perm)
               for perm in ('cfblog.change_content', 'cfblog.can_publish'))


def can_publish_content(user):
    return user.has_perm('cfblog.can_publish')


def user_passes_test(test_func=None):
    """
    Similar to user_passes_test in django.contrib.auth.decorators.
    Instead of redirecting it just raises PermissionDenied.

    :param test_func: Should take a User object and return a bool
    :type test_func: function
    """
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):
            if test_func(request.user):
                return view_func(request, *args, **kwargs)
            else:
                raise PermissionDenied
        return _wrapped_view
    return decorator



# coding=utf-8
import datetime
import json

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.test import SimpleTestCase, TestCase
from django.test.utils import override_settings
from django.utils.decorators import method_decorator

from .models import Content
from .utils import NAMESPACE_DELIMITER, parse_cms_template


class BaseTests(SimpleTestCase):
    cms_context = {}
    template_context = None

    def setUp(self):
        self.html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                        hello
                    </h1>
                </body>
            </html>
        """

    @property
    def output(self):
        return parse_cms_template(self.html, self.cms_context, public=True,
                                  template_context=self.template_context)


class BasicTagTests(BaseTests):

    def test_empty_context(self):
        self.cms_context = {}
        self.assertHTMLEqual(self.html, self.output)

    def test_data_cms_content(self):
        self.cms_context = {"h1": "foo"}
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                        foo
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_data_cms_attr(self):
        self.cms_context = {"id": "new_id"}
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="new_id">
                        hello
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_data_cms_replace(self):
        self.html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig"
                        data-cms-replace>
                    <div data-cms-content="content" data-cms-attr="id:div_id"
                        id="replaced">
                            new content
                    </div>
                    </h1>
                </body>
            </html>
            """
        self.cms_context = {}
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                </body>
            </html>
            """,
            self.output
        )
        self.cms_context = {'h1': '<div>replaced h1</div>'}
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <div>replaced h1</div>
                </body>
            </html>
            """,
            self.output
        )


class MultiLevelTagTests(BaseTests):
    def test_single_level_data(self):
        self.cms_context = {'h1': '<div data-cms-content="text"></div>'}
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="text"></div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_multi_level_data(self):
        self.cms_context = {
            'h1': '<div data-cms-content="text"></div>',
            'h1' + NAMESPACE_DELIMITER + 'text': 'replaced'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="text">replaced</div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_single_level_complete_data(self):
        self.cms_context = {
            'h1': '<div data-cms-attr="id:div_id" id="default">hi</div>',
            'h1' + NAMESPACE_DELIMITER + 'div_id': 'replaced'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-attr="id:div_id" id="replaced">hi</div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_multi_level_complete_data(self):
        self.cms_context = {
            'h1': (
                '<div data-cms-content="content" '
                'data-cms-attr="id:div_id" id="default">hi</div>'
            ),
            'h1' + NAMESPACE_DELIMITER + 'div_id': 'replaced',
            'h1' + NAMESPACE_DELIMITER + 'content': 'new content'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="content" data-cms-attr="id:div_id"
                        id="replaced">
                            new content
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )


class CMSTests(BaseTests):
    def test_snippet_insertions(self):
        self.cms_context = {
            'h1': '<div data-cms-include="namespace:cms_templates/snippet">'
                  '</div>'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet"
                        data-cms-namespace="namespace">
                            hehehe
                            <div data-cms-content="md:text">Hi</div>
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_snippet_insertions_data(self):
        self.cms_context = {
            'h1': '<div data-cms-include="namespace:cms_templates/snippet">'
                  '</div>',
            NAMESPACE_DELIMITER.join(
                ('h1', 'namespace', 'text')
            ): '##Heading##'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet"
                        data-cms-namespace="namespace">
                            hehehe
                            <div data-cms-content="md:text">
                                <h2>Heading</h2>
                            </div>
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

        self.cms_context = {
            'h1': '<div '
                  'data-cms-include="namespace:cms_templates/snippet_2">'
                  '</div>',
            NAMESPACE_DELIMITER.join(
                ('h1', 'namespace', 'text2')
            ): '##Heading##'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet_2"
                        data-cms-namespace="namespace">
                        <div data-cms-content="md:text2"><h2>Heading</h2></div>
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_template_context(self):
        self.cms_context = {
            'h1': '<div data-cms-include="namespace:cms_templates/snippet">'
                  '</div>',
            NAMESPACE_DELIMITER.join(
                ('h1', 'namespace', 'text')
            ): '##Heading##'
        }
        self.template_context = {
            'greeting': 'hello'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet"
                        data-cms-namespace="namespace">
                            hello
                            <div data-cms-content="md:text">
                                <h2>Heading</h2>
                            </div>
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )

    def test_markdown(self):
        self.html = """
        <html>
            <head>
            </head>
            <body>
                <h1 data-cms-content="md:h1" data-cms-attr="id:id" id="orig">
                    hello
                </h1>
            </body>
        </html>
        """
        self.cms_context = {
            'h1': '##Heading##\n'
                  '<div '
                  'data-cms-include="namespace:cms_templates/snippet_2">'
                  '</div>',
            NAMESPACE_DELIMITER.join(
                ('h1', 'namespace', 'text2')
            ): '##Heading##'
        }
        self.assertHTMLEqual(
            """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="md:h1" data-cms-attr="id:id"
                        id="orig">
                    <h2>Heading</h2>
                    <div data-cms-include="cms_templates/snippet_2"
                        data-cms-namespace="namespace">
                            <div data-cms-content="md:text2">
                                <h2>Heading</h2>
                            </div>
                    </div>
                    </h1>
                </body>
            </html>
            """,
            self.output
        )


class RegressionTests(TestCase):

    @classmethod
    def setUpTestData(cls):
        User = get_user_model()
        editor_credentials = {
            User.USERNAME_FIELD: 'editor',
            'password': 'test123'
        }
        author_credentials = {
            User.USERNAME_FIELD: 'author',
            'password': 'test123'
        }
        editor = User.objects.create_user(**editor_credentials)
        author = User.objects.create_user(**author_credentials)
        # We are explicitly setting is_active because of django 1.8
        editor.is_active = True
        editor.save()
        author.is_active = True
        author.save()
        edit_permission = Permission.objects.get(
            codename='change_content',
            content_type__app_label='cfblog'
        )
        publish_permission = Permission.objects.get(
            codename='can_publish',
            content_type__app_label='cfblog'
        )
        author.user_permissions.add(publish_permission)
        editor.user_permissions.add(edit_permission)
        cls.author = author
        cls.editor = editor
        cls.author_client = cls.client_class()
        cls.author_client.login(**author_credentials)
        cls.editor_client = cls.client_class()
        cls.editor_client.login(**editor_credentials)

    def setUp(self):
        cms_page, _ = Content.objects.get_or_create(
            url='/test-golbfc/',
            template="cms_templates/template_1.html",
            category_id=1,
            author=self.author
        )
        self.cms_page = cms_page

    def test_unpublished_url(self):
        self.cms_page.status = Content.DRAFT
        self.cms_page.save()
        response = self.client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 404)
        response = self.editor_client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 200)
        response = self.author_client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 200)

    def test_published_url(self):
        self.cms_page.status = Content.PUBLIC
        self.cms_page.save()
        response = self.client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 200)
        response = self.editor_client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 200)
        response = self.author_client.get(self.cms_page.url)
        self.assertEqual(response.status_code, 200)

    @method_decorator(override_settings(ROOT_URLCONF=()))
    def test_custom_url_conf(self):
        # response should be served by the middleware
        self.test_published_url()
        self.test_unpublished_url()

    def test_editor_cannot_publish(self):
        response1 = self.editor_client.post(
            path='/cms/ajax/save/publish/',
            data={},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.assertEqual(response1.status_code, 403)

    def test_author_can_publish(self):
        response1 = self.author_client.post(
            path='/cms/ajax/save/publish/',
            data={},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.assertNotEqual(response1.status_code, 403)

    def test_versioning(self):

        content = json.dumps({
            "body": "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
            "body_attr": "random data",
            "decrp": "Hi!!",
            "title": "Hello World!!!"
        })
        draft_time = self.cms_page.modified_on + datetime.timedelta(minutes=5)
        data = {
            'auth_data': content,
            'draft_modified': draft_time.isoformat(),
            'cms_page_id': self.cms_page.id
        }

        response1 = self.author_client.post(
            path='/cms/ajax/save/publish/',
            data=data,
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.assertEqual(response1.status_code, 200)
        self.assertTrue(json.loads(response1.content)['success'])

        draft_time = self.cms_page.modified_on - datetime.timedelta(minutes=5)
        data['draft_modified'] = draft_time.isoformat()
        response2 = self.author_client.post(
            path='/cms/ajax/save/publish/',
            data=data,
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.assertEqual(response2.status_code, 200)
        self.assertFalse(json.loads(response2.content)['success'])
        self.assertTrue(json.loads(response2.content)['draft_error'])



# coding=utf-8
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
from django.db.models import Q
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from jsonfield import JSONField
from tagging.fields import TagField

from .managers import ContentManager
from .utils import dum_request, parse_cms_template
from .validators import validate_and_get_template, validate_url_path


class Category(models.Model):
    """Category model."""
    title = models.CharField(_('title'), max_length=100,
                             unique=True, db_index=True)
    description = models.TextField(_('description'))
    is_static = models.BooleanField(default=False, db_index=True)

    class Meta(object):
        verbose_name_plural = _('categories')
        ordering = ('title',)

    def __unicode__(self):
        return self.title


def cms_authors():
    User = get_user_model()
    users = User.objects.filter(
        Q(user_permissions__codename='change_content',
          user_permissions__content_type__app_label='cfblog') |
        Q(groups__permissions__codename='change_content',
          groups__permissions__content_type__app_label='cfblog') |
        Q(is_superuser=True)
    ).distinct()

    return {'pk__in': users.values('pk')}


class Content(models.Model):
    DRAFT = 1
    PUBLIC = 2
    STATUS_CHOICES = (
        (DRAFT, _('Draft')),
        (PUBLIC, _('Public')),
    )
    url = models.CharField(_('url path'), max_length=255, db_index=True,
                           unique=True, validators=[validate_url_path])
    template = models.CharField(verbose_name=_('template'), max_length=255,
                                validators=[validate_and_get_template])
    status = models.IntegerField(
        _('status'), choices=STATUS_CHOICES, default=DRAFT
    )
    publish = models.DateTimeField(_('publish'), default=timezone.now)
    title = models.CharField(_('title'), max_length=200)
    category = models.ForeignKey('Category')
    tags = TagField()
    tease = models.TextField(
        _('tease'), blank=True,
        help_text=_('Concise text suggested. Does not appear in RSS feed.')
    )
    notes_todos = models.TextField(_('Notes and Todos'), blank=True)
    auth_data = JSONField(
        verbose_name=_('author data'), default=dict, blank=True,
        help_text=_("Don't edit this unless you know what this means")
    )
    public_data = JSONField(
        verbose_name=_('public data'), default=dict, blank=True,
        help_text=_("Don't edit this unless you know what this means")
    )
    author = models.ForeignKey(settings.AUTH_USER_MODEL,
                               limit_choices_to=cms_authors)
    created_on = models.DateTimeField(_('created on'), auto_now_add=True)
    modified_on = models.DateTimeField(_('modified on'), auto_now=True)
    objects = ContentManager()

    class Meta(object):
        ordering = ('-publish',)
        get_latest_by = 'publish'
        permissions = (('can_publish', "Can Publish Posts"),)

    @property
    def is_public(self):
        return self.status == self.PUBLIC

    def publish_cms_content(self):
        self.public_data = self.auth_data
        self.status = self.PUBLIC
        self.save()

    def __unicode__(self):
        return self.title

    def get_previous_post(self):
        return self.get_previous_by_publish(
            Q(status__gte=self.PUBLIC), ~Q(category__is_static=True)
        )

    def get_next_post(self):
        return self.get_next_by_publish(
            Q(status__gte=self.PUBLIC), ~Q(category__is_static=True)
        )

    def get_absolute_url(self):
        return self.url

    def get_public_html(self, request=dum_request,
                        template_context=None, using=None):
        # This is only recommended for internal usages
        # where you want to extract the html content
        # without publishing the page
        template_context = template_context or {}
        template_context['cms_content'] = self
        template = validate_and_get_template(self.template, using=using)
        content = template.render(context=template_context, request=request)
        return parse_cms_template(
            html=content, cms_context=self.public_data, public=True,
            request=request, template_context=template_context
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import warnings
import sys

from django.core.management.color import no_style
from django.db import migrations


def create_static_page_category(apps, schema_editor):
    Category = apps.get_model('cfblog', 'Category')
    sys.stdout.write('Creating the static pages category')
    if Category.objects.filter(id=1):
        warnings.warn(
            "Entry for static pages already exists."
        )
    else:
        reset_sequence = True
        if Category.objects.count():
            reset_sequence = False

        Category.objects.create(
            id=1,
            title='Static Page',
            description='All the standalone pages should be under this category'
        )

        if reset_sequence:
            connection = schema_editor.connection
            sql_statements = connection.ops.sequence_reset_sql(no_style(), [Category])
            operation = migrations.RunSQL(sql=sql_statements)
            operation.database_forwards('cfblog', schema_editor, None, None)


def delete_static_page_category(apps, schema_editor):
    Category = apps.get_model('cfblog', 'Category')
    categories = Category.objects.filter(id=1)
    sys.stdout.write('Deleting the static pages category')
    if categories:
        categories.delete()
    else:
        warnings.warn(
            "Entry for static pages does not exist."
        )


class Migration(migrations.Migration):

    dependencies = [
        ('cfblog', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(
            code=create_static_page_category,
            reverse_code=delete_static_page_category
        )
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cfblog', '0002_create_static_page_category'),
    ]

    operations = [
        migrations.AlterField(
            model_name='content',
            name='auth_data',
            field=jsonfield.fields.JSONField(default=dict, help_text="Don't edit this unless you know what this means", verbose_name='author data', blank=True),
        ),
        migrations.AlterField(
            model_name='content',
            name='public_data',
            field=jsonfield.fields.JSONField(default=dict, help_text="Don't edit this unless you know what this means", verbose_name='public data', blank=True),
        ),
        migrations.AlterField(
            model_name='content',
            name='publish',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='publish'),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

try:
    long_description = open('README.rst', 'rt').read()
except Exception:
    long_description = ""

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name="cfblog",
    description="Awesome CMS for Django",
    long_description=long_description,

    version="0.2.10",
    author='Vinay Karanam, Ashish Nayan',
    author_email="vinayinvicible@gmail.com, nayanashish@gmail.com",

    url='https://github.com/Coverfox/cfblog/',
    license='BSD',

    install_requires=requirements,
    packages=find_packages(),
    include_package_data=True,

    zip_safe=True,
)



# coding=utf-8
from django.contrib import admin

from .models import Category, Content


class CmsPageCategoryAdmin(admin.ModelAdmin):
    pass

admin.site.register(Category, CmsPageCategoryAdmin)


class ContentAdmin(admin.ModelAdmin):
    list_display = ('url', 'title', 'publish', 'status', 'link')
    list_filter = ('publish', 'category', 'status', 'author')
    search_fields = ('title', 'url', 'template')
    readonly_fields = ('created_on', 'modified_on')

    def link(self, obj):
        return "<a target='_blank' href='{}'>View on site</a>".format(
            obj.get_absolute_url()
        )
    link.allow_tags = True

admin.site.register(Content, ContentAdmin)



# coding=utf-8
from django.conf import settings as django_settings

from . import global_settings


class CustomSettings(object):
    def __getattr__(self, item):
        try:
            return getattr(django_settings, item)
        except:
            if item.isupper() and hasattr(global_settings, item):
                return getattr(global_settings, item)
            raise

settings = CustomSettings()

# The CustomSettings class is no more needed, so remove it from the namespace.
del CustomSettings



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.db.models import Q
from django.conf import settings
from django.core.management.sql import emit_post_migrate_signal


def emit_post_migrate_to_avoid_breaking_continuity(schema_editor):
    """
     This is so that running migrations together doesn't break, because
     Django adds Permissions in post_migrate signal
    """
    db_alias = schema_editor.connection.alias
    try:
        emit_post_migrate_signal(0, False, db_alias)
    except TypeError:  # Django < 1.9
        try:
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)


def add_publish_permission(apps, schema_editor):
    app = settings.AUTH_USER_MODEL.split('.')[0]
    User = apps.get_model(app, 'User')
    users = User.objects.filter(
        user_permissions__codename='change_content',
        user_permissions__content_type__app_label='cfblog'
    )

    Group = apps.get_model('auth', 'Group')
    groups = Group.objects.filter(
        permissions__codename='change_content',
        permissions__content_type__app_label='cfblog'
    )

    emit_post_migrate_to_avoid_breaking_continuity(schema_editor)
    Permission = apps.get_model('auth', 'Permission')
    permission = Permission.objects.get(codename='can_publish')

    for user in users.iterator():
        user.user_permissions.add(permission)

    for group in groups.iterator():
        group.permissions.add(permission)


class Migration(migrations.Migration):

    dependencies = [
        ('cfblog', '0004_auto_20160406_1820'),
    ]

    operations = [
        migrations.RunPython(
            code=add_publish_permission,
            reverse_code=migrations.RunPython.noop
        )
    ]

    def __init__(self, *args, **kwargs):
        super(Migration, self).__init__(*args, **kwargs)
        from django.apps import apps
        # emit_post_migrate_signal tries to create permissions and sites
        # so sites and auth app should be migrated before this migration
        # we also need to check if they are present in INSTALLED_APPS
        for app in ('sites', 'auth'):
            if app in apps.app_configs:
                self.dependencies.append((app, '__first__'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cfblog', '0003_auto_20151126_1212'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='content',
            options={'ordering': ('-publish',), 'get_latest_by': 'publish', 'permissions': (('can_publish', 'Can Publish Posts'),)},
        ),
    ]
